/*
 * $Id: Board.java,v 1.2 1998/04/07 12:27:15 vmallet Exp $
 *
 *
 * $Log: Board.java,v $
 * Revision 1.2  1998/04/07 12:27:15  vmallet
 * Added the putWord() method moved from ServerBoard.
 *
 * Revision 1.1  1998/04/03 16:28:25  poivert
 * Initial revision
 *
 *
 */ 

package scrobble;

import java.util.*;
import scrobble.square.*; 

/**
 * The <code>Board</code> class represents the board used in the 
 * Scrabble game. It is configured from a <code>BoardShape</code> 
 * object. It is made up of squares and implements the main methods
 * to verify and to put the words on the board  
 * 
 * @version $Revision: 1.2 $
 * @author  Frederique Poivert (poivert@enst.fr)
 */
public class Board { 

  /**
   * Shape of the Board
   */
  BoardShape shape;

  /**
   * Matrix of Squares
   */
  Square squares[][];



  // Constructor


  /**
   * Creates a new Board with the given shape
   * @param shape the shape of the new board created
   */
  public Board(BoardShape shape) {
    this.shape = shape;
    int h = shape.getHeight();
    int w = shape.getWidth();
    squares = new Square[h][w];

    // double boucle pour allouer les cases par defaut :

    for (int i = 0; i < h; i++)
      for (int j = 0; j < w; j++)
	squares[i][j] = shape.getDefaultSquare();

    // Allocation des cases speciales donnees dans shape :

    for (Enumeration e = shape.getSquareList().elements(); e.hasMoreElements(); ) {
      SquarePositioner sp = (SquarePositioner) e.nextElement();
      squares[sp.getWidthPosition()][sp.getHeightPosition()] = sp.getSquare();
    }
    
    
  }



  // Methods


  /**
   * Return the shape of the board
   * @return the shape
   */
  public BoardShape getShape() {
    return shape;
  }





  /**
   * Returns the list of words on the board
   * @return the Vector of words
   */
  public Vector getWords() {
    return null;
    // a completer
  }


  /**
   * Returns the list of letters on the board
   * @return the Vector of letters
   */
  public Vector getLetters() {
    return null;
    // a completer
  }



  /**
   * Empties the board
   */
  public void empty() {
    // dit aux cases de virer leurs lettres
  }



  // Faire deux methodes : tryAndFit vertical et 
  //                       tryAndFit horizontal


  /**
   * Tries to fit a word to the board according to the given position 
   * @param wordp the WordPositioner containing the word and the position of the word
   * @return the list of the resulting words on the board.
   * @exception IllegalPositionException if it was impossible to fit the given word to the board according to the given position. 
   */

  Vector tryAndFit(WordPositioner wordp) throws IllegalPositionException {
    // retourne un vector de WordPositioner
    // a completer

    Vector vw         = new Vector();
    int dir           = wordp.getDirection();
    int w0            = wordp.getWidthPosition();
    int h0            = wordp.getHeightPosition();
    int nb            = wordp.getWord().getVectorWord().size();
    int hdim          = getShape().getHeight();
    int wdim          = getShape().getWidth();
    // Vector interWord  = new Vector(); 
    Word w            = new Word ();
    WordPositioner wp = new WordPositioner();

    // Etapes de la verification "mecanique" :
    // First test : aux deux extremites du mot doit se trouver soit une case vide, soit une case neant. 
    // Second test : a chaque element du mot doit correspondre soit une case vide (mais pas une case neant) 
    //    que l'on va pouvoir remplir avec la bonne lettre, soit une case pleine contenant la bonne lettre.
    // Third part : Si on est arrive jusque la, c'est que la verification "mecanique" a reussi. Reste a 
    ///   retourner tous les mots formes, le premier etant celui du WordPositioner place en parametre de tryAndFit.
    // Les autres mots nouvellement formes ne peuvent etre trouves qu'a partir des elements Letter du mot (celui du 
    //    WordPositioner place en parametre de tryAndFit). En effet, les elements SquarePositioner representent des 
    //    lettres deja posees sur le Board, et donc ne peuvent generer des mots nouveaux.
    

    // first test, cas horizontal :

    if (dir == WordPositioner.HORIZONTAL) {
      Square lSq, rSq;
      try {
	lSq = squares[h0][w0-1];   //case a l'extreme gauche du mot horiz
      }
      catch (ArrayIndexOutOfBoundsException e) {
	lSq = null;
      }

      try {
	rSq = squares[h0][w0+nb];   //case a l'extreme droite du mot horiz
      }
      catch (ArrayIndexOutOfBoundsException e) {
	rSq = null;
      }

      if ( ( (lSq != null) && (lSq.getLetter() != null) ) || 
	   ( (rSq != null) && (rSq.getLetter() != null) ) ) 
	throw new IllegalPositionException();
	


      // second test, cas horizontal :

      for (int j = 0; j < nb; j++ ) {

	try {
	  if (squares[h0][w0+j] == null) throw new IllegalPositionException();
	}
	catch (ArrayIndexOutOfBoundsException e) {
	  throw new IllegalPositionException();
	}

	Object obj = wordp.getWord().getVectorWord().elementAt(j);

	if (obj instanceof SquarePositioner) {

	  //@@ a rendre plus joli

	  SquarePositioner sp = (SquarePositioner) obj;
	  SquarePositioner boardSp = new SquarePositioner(w0+j, h0);
	  if ( !(sp.comparePositions(boardSp)) ) throw new IllegalPositionException();

	} else if (obj instanceof Letter) {
	  if ( (squares[h0][w0+j]).getLetter() != null) throw new IllegalPositionException();
	}

      }

      
      // third part, cas horizontal :

      // Si on est arrive jusque la, c'est que la verification "mecanique" a reussi.
      // Reste a retourner tous les mots formes, le premier etant celui du WordPositioner 
      //     place en parametre de tryAndFit.
      // Les autres mots nouvellement formes ne peuvent etre trouves qu'a partir des elements
      //     Letter du mot (celui du WordPositioner place en parametre de tryAndFit). En effet, 
      //     les elements SquarePositioner representent des lettres deja posees sur le Board, et 
      //     donc ne peuvent generer des mots nouveaux.
	
      vw.addElement(wordp) ;

      for (int j = 0; j < nb; j++ ) {

	Object o = wordp.getWord().getVectorWord().elementAt(j);

	if (o instanceof Letter) {
	  Letter l = (Letter) o;

	  // Une lettre du mot horizontal  peut engendrer un autre mot si l'une des cases au-dessus d'elle ou au-dessous d'elle contient une lettre.
 
	  if ( ( ((h0-1) >= 0) && 
		 (squares[h0-1][w0+j] != null) && 
		 ( (squares[h0-1][w0+j]).getLetter() != null) ) ||
	       ( ((h0+1) <= (hdim-1)) &&
		 (squares[h0+1][w0+j] != null) && 
		 ( (squares[h0+1][w0+j]).getLetter() != null) ) ) {
 

	    /////////////////////////////////////////////////////////
	    Word ww = new Word();
	    ww.addLetter(l);
	    WordPositioner wwp = new WordPositioner();
	    wwp.setDirection(WordPositioner.VERTICAL);
	    ////////////////////////////////////////////////////////

	    // on recherche les lettres au-dessus :

	    int i = 1;
	    while ( ((h0-i) >=0) && 
		    (squares[h0-i][w0+j] != null) && 
		    ( (squares[h0-i][w0+j]).getLetter() != null) ) {
	    
	      // On ajoute une letttre en debut du mot de wwp :

	      /////////////////////////////////////////////////////////
	      ww.insertLetterAtFirstPosition( (squares[h0-i][w0+j]).getLetter() ) ;
	      i++;
	    }

	    wwp.setWidthPosition(w0+j);
	    wwp.setHeightPosition(h0-i+1);
	    ////////////////////////////////////////////////////////////

	    // on recherche les lettres au-dessous :

	    i = 1;
	    while ( ((h0+i) <= (hdim-1)) && 
		    (squares[h0+i][w0+j] !=null) && 
		    ( (squares[h0+i][w0+j]).getLetter() !=null) ) {
	  
	      // On ajoute une lettre a la fin du mot de wwp 
      
	      ww.addLetter( (squares[h0+i][w0+j]).getLetter() ); 

	      i++;
	    }

	    wwp.setWord(ww);
	  
	    // on ajoute ce nouveau WordPositioner a la liste des mots formes :

	    vw.addElement(wwp) ;

	    ///////////////////////////////////////////////////

	  } 

	    
	}


      } 


	

      // first test, cas vertical :

    } else if (dir == WordPositioner.VERTICAL) {
      Square hSq, uSq; 
      try {
	hSq = squares[h0-1][w0];    //case juste au-dessus du mot vertical
      }
      catch (ArrayIndexOutOfBoundsException e) {
	hSq = null;
      }

      try {
	uSq = squares[h0+nb][w0];   //case juste au-dessous du mot vertical
      }
      catch (ArrayIndexOutOfBoundsException e) {
	uSq = null;
      }
      if ( ( (hSq != null) && (hSq.getLetter() != null) ) || 
	   ( (uSq != null) && (uSq.getLetter() != null) ) )
	throw new IllegalPositionException();
      

      // second test, cas vertical :

      for (int i = 0; i < nb; i++ ) {

	try {
	  if (squares[h0+i][w0] == null) throw new IllegalPositionException();
	}
	catch (ArrayIndexOutOfBoundsException e) {
	  throw new IllegalPositionException();
	}
	
	Object obj = wordp.getWord().getVectorWord().elementAt(i);
	
	if (obj instanceof SquarePositioner) {

	  //@@ a rendre plus joli

	  SquarePositioner sp = (SquarePositioner) obj;
	  SquarePositioner boardSp = new SquarePositioner(w0, h0+i);
	  if ( !(sp.comparePositions(boardSp)) ) throw new IllegalPositionException();
	  
	} else if (obj instanceof Letter) {
	  if ( (squares[h0+i][w0]).getLetter() != null) throw new IllegalPositionException();
	}
	
      }

      // third part, cas vertical :

      // Si on est arrive jusque la, c'est que la verification "mecanique" a reussi.
      // Reste a retourner tous les mots formes, le premier etant celui du WordPositioner place en parametre de tryAndFit.
      // Les autres mots nouvellement formes ne peuvent etre trouves qu'a partir des elements Letter du mot (celui du WordPositioner place en parametre de tryAndFit). En effet, les elements SquarePositioner representent des lettres deja posees sur le Board, et donc ne peuvent generer des mots nouveaux.
	
      vw.addElement(wordp) ;

      for (int i = 0; i < nb; i++ ) {

	Object o = wordp.getWord().getVectorWord().elementAt(i);

	if (o instanceof Letter) {
	  Letter l = (Letter) o;

	  // Une lettre du mot vertical  peut engendrer un autre mot si l'une des cases a sa droite ou a sa gauche contient une lettre.
 
	  if ( ( ((w0-1) >= 0) && 
		 (squares[h0+i][w0-1] !=null) && 
		 ( (squares[h0+i][w0-1]).getLetter() !=null) ) ||
	       ( ((w0+1) <= (wdim-1)) &&
		 (squares[h0+i][w0+1] !=null) && 
		 ( (squares[h0+i][w0+1]).getLetter() !=null) ) ) {


	    ////////////////////////////////////////// 
	    Word ww = new Word();
	    ww.addLetter(l);
	    WordPositioner wwp = new WordPositioner();
	    wwp.setDirection(WordPositioner.HORIZONTAL);
	

	    // on recherche les lettres a gauche :

	    int j = 1;
	    while ( ((w0-j) >=0) && 
		    (squares[h0+i][w0-j] !=null) && 
		    ( (squares[h0+i][w0-j]).getLetter() !=null) ) {
	    
	      // On ajoute une lettre en debut de mot :

	      ww.insertLetterAtFirstPosition( (squares[h0+i][w0-j]).getLetter() );

	      j++;
	    }

	    wwp.setWidthPosition(w0-j+1);
	    wwp.setHeightPosition(h0+i);
	

	    // on recherche les lettres a droite :

	    j = 1;
	    while ( ((w0+j) <= (wdim-1)) && 
		    (squares[h0+i][w0+j] !=null) && 
		    ( (squares[h0+i][w0+j]).getLetter() !=null) ) {
	  
	      //@@ Ces instructions sont-elles correctes pour ajouter un element a la fin du mot de wp ?
	    
	      ww.addLetter( (squares[h0+i][w0+j]).getLetter() );
	   

	      j++;
	    }

	    wwp.setWord(ww);

	
	    // on ajoute ce nouveau WordPositioner a la liste des mots formes :

	    vw.addElement(wwp) ;


	  } // else interWord.addElement(new Integer(0));

	}      


      }
    

      

    }


    return vw;

  } // tryAndFit


  /**
   * Puts the word on the board
   * @param wordp the WordPositioner containing the word and the position of the word
   */

  public void putWord(WordPositioner wordp) {

    int nb = wordp.getWord().getVectorWord().size();
    int w0 = wordp.getWidthPosition();
    int h0 = wordp.getHeightPosition();
    
    if (wordp.getDirection() == WordPositioner.VERTICAL) {

      for (int i = 0; i < nb; i++ ) {
	  
	Object obj = wordp.getWord().getVectorWord().elementAt(i);
	  
	if (obj instanceof Letter) {

	  Letter l = (Letter) obj;
	  (squares[h0+i][w0]).putLetter(l);
	  
	} 
      
      }
    
    } else if (wordp.getDirection() == WordPositioner.HORIZONTAL) { 

      for (int j = 0; j < nb; j++ ) {
	  
	Object obj = wordp.getWord().getVectorWord().elementAt(j);
	  
	if (obj instanceof Letter) {

	  Letter l = (Letter) obj;
	  (squares[h0][w0+j]).putLetter(l);
	  
	} 
      
      }
    }

  }  


  /**
   * Transmits over the Net 
   */  
  public void transmit() {
    // parametre a voir...
  }


  /**
   * Receive from the Net
   */
  public void receive() {
    // parametre a voir...
  }


  /**
   * Dumps the board to the console
   */  
  public void dump() {
    System.out.println(" Height of the board : " + shape.getHeight());
    System.out.println(" Width of the board : " + shape.getWidth());
    System.out.println(" Board : ");
    for (int i = 0; i< shape.getHeight(); i++){
      for (int j = 0; j< shape.getWidth(); j++)
	if ((squares[i][j]) == null)
	  System.out.print(0 + "      ");
	else
	  System.out.print((squares[i][j]).getLetter() + "      ");
      System.out.println("");
    }
  }


  public static void main(String args[]) { 
    Vector v = new Vector();
    Dico mondico = new Dico("words");
    v.addElement(new Integer(5));
    v.addElement(new Integer(2));
    v.addElement(new Integer(90));
    System.out.println("Taille de mon vecteur : " + v.size());
    int i1 = ((Integer) v.firstElement()).intValue();
    int i11 = ((Integer) v.elementAt(0)).intValue();
    int i2 = ((Integer) v.elementAt(1)).intValue();
    int i3 = ((Integer) v.elementAt(2)).intValue();
    System.out.println("Element 0 : " + i1);
    System.out.println("Element 0 : " + i11);
    System.out.println("Element 1 : " + i2);
    System.out.println("Element 2 : " + i3);

    ServerDoubleLetterSquare dsq = new ServerDoubleLetterSquare();
    SquarePositioner sp = new SquarePositioner(3, 2, dsq);  
    Vector squareList = new Vector();
    squareList.addElement(sp);
    ServerSquare defaultSquare = new ServerSquare();
    BoardShape shape = new BoardShape(5, 5, defaultSquare, squareList);
    ServerBoard board = new ServerBoard(shape);
    board.dump();

    Word w = new Word();
    w.addLetter(new Letter('a'));
    w.addLetter(new Letter('r'));
    w.addLetter(new Letter('b'));
    w.addLetter(new Letter('r'));
    w.addLetter(new Letter('e'));
    WordPositioner wp = new WordPositioner(0, 0, w, WordPositioner.VERTICAL);
    board.putWord(wp);
    System.out.println(" ");
    board.dump();


    Word w1 = new Word();
    //@@ 
    w1.getVectorWord().addElement(new SquarePositioner(0,0,board.squares[0][0]));
    w1.addLetter(new Letter('p'));
    w1.addLetter(new Letter('p'));
    w1.addLetter(new Letter('l'));
    w1.addLetter(new Letter('e'));
    WordPositioner wp1 = new WordPositioner(0,0,w1,WordPositioner.HORIZONTAL);
    try {
      //board.tryAndFit(wp1);
       board.verifyAndCount(wp1, mondico); 
       board.putWord(wp1);
       System.out.println(" ");
       board.dump();
     }
     catch (IllegalPositionException e) {
       System.out.println(" Error of position");
     }

     catch (IllegalWordException e) {
       System.out.println(" Error of word");
     }

    Word w3 = new Word();
    //@@ 
    w3.getVectorWord().addElement(new SquarePositioner(0,1,board.squares[1][0]));
    w3.addLetter(new Letter('o'));
    w3.addLetter(new Letter('o'));
    w3.addLetter(new Letter('t'));
    // w3.addLetter(new Letter('s'));
    WordPositioner wp3 = new WordPositioner(0,1,w3,WordPositioner.HORIZONTAL);
    try {
      //board.tryAndFit(wp2);
      board.verifyAndCount(wp3, mondico); 
      board.putWord(wp3);
       System.out.println(" ");
       board.dump();
     } catch (IllegalPositionException e) {
       System.out.println(" Error of position");
     } catch (IllegalWordException e) {
       System.out.println(" Error of word : " + e.getMessage());
  }

    
    
  }


}
