/*
 * $Id: BoardShape.java,v 1.2 1998/04/06 09:46:53 vmallet Exp $
 *
 *
 * $Log: BoardShape.java,v $
 * Revision 1.2  1998/04/06 09:46:53  vmallet
 * Now implements Serializable.
 *
 * Revision 1.1  1998/04/03 16:30:56  poivert
 * Initial revision
 *
 *
 */ 

package scrobble;

import java.io.*; 
import java.util.*;

import scrobble.square.*; 


/**
 * The <code>BoardShape</code> code is used to configure the <code>Board</code>
 * objects created in the game. It contains the board dimensions, the default
 * square object, and a list of special squares.
 *
 * @version $Revision: 1.2 $
 * @author  Frederique Poivert (poivert@enst.fr)
 */


public class BoardShape implements Serializable {


  /**
   * Height of the Board
   */
  int height;

  /**
   * Width of the Board
   */
  int width;

  /**
   * Default Square
   */
  Class defaultSquareClass;

  /**
   * List of SquarePositioners containing the square and the position of the square on the board
   */
  Vector squareList;


  
  // Constructor

  /**
   * Creates a new BoardShape with the given height,width,defaultSquare and squareList
   * @param height the height of the new BoardShape being created
   * @param width the width of the new BoardShape
   * @param defaultSquare the defaultSquare of the new BoardShape
   * @param squareList the squareList of the new BoardShape
   */
  public BoardShape(int height, int width, Square defaultSquare, Vector squareList) {
    this.height = height;
    this.width = width;
    this.defaultSquareClass = defaultSquare.getClass();
    this.squareList = squareList;
  }



  // Methods



  /**
   * Modifies the height of the BoardShape
   * @param newHeight the new height of the BoardShape
   */
  public void setHeight(int newHeight) {
    height = newHeight;
  }


  /**
   * Returns the height of the BoardShape
   * @return the height
   */
  public int getHeight() {
    return height;
  }


  /**
   * Modifies the width of the BoardShape
   * @param newWidth the new width of the BoardShape
   */
  public void setWidth(int newWidth) {
    width = newWidth;
  }


  /**
   * Returns the width of the BoardShape
   * @return the width
   */
  public int getWidth() {
    return width;
  }


  /**
   * Modifies the defaultSquare of the BoardShape
   * @param newDefaultSquare the new defaultSquare of the BoardShape
   */
  public void setDefaultSquare(Square newDefaultSquare) {
    defaultSquareClass = newDefaultSquare.getClass();
  }


  /**
   * Returns the defaultSquare of the BoardShape
   * @return the defaultSquare
   */
  public Square getDefaultSquare() {
    Square s;
    try {
      s = (Square) defaultSquareClass.newInstance();
    }
    catch (Exception e) {
      s = null;
    }
    return s;
  }
  


  /**
   * Modifies the squareList of the BoardShape
   * @param newSquareList the new squareList of the BoardShape
   */
  public void setSquareList(Vector newSquareList) {
    squareList = newSquareList;
  }


  /**
   * Returns the squareList of the BoardShape
   * @return the squareList
   */
  public Vector getSquareList() {
    return squareList;
  }



}
