/*
 * $Id: ClientBag.java,v 1.5 1998/04/06 17:32:51 vmallet Exp $
 *
 *
 * $Log: ClientBag.java,v $
 * Revision 1.5  1998/04/06 17:32:51  vmallet
 * Added setLetterNbr(),  updateLabel().
 * Fixed a few bugs.
 * Lot of changes to frameReceived().
 *
 * Revision 1.4  1998/04/06 09:44:05  vmallet
 * Fixed the getLetterNbr() method.
 * Rewrote the getComponent() method.
 *
 * Revision 1.3  1998/04/05 17:45:00  granier
 * Insertion of debug labels
 *
 * Revision 1.2  1998/04/05 12:52:31  granier
 * If cheat or not different frames are received
 *
 * Revision 1.1  1998/04/05 11:40:53  granier
 * Initial revision
 *
 *
 */


package scrobble;

import game.*;


import java.util.*;
import java.awt.*;
import com.sun.java.swing.*;
import com.sun.java.swing.border.*;

/**
 * This class is the client version of the class <code>Bag</code>, mostly 
 * devoted for the graphic interface.
 * 
 * @version $Revision: 1.5 $
 * @author  Anne Granier (granier@enst.fr)
 */


public class ClientBag extends Bag implements GameLinkListener, GameLinkConstants {

  /**
   * if true, enables the printing of debug informations on the console.
   */  
  static boolean debug = true;

  /**
   * The label which contains the score of Player;
   */
  protected JLabel baglabel;

  protected int bagSize;

  Vector letters;

  //Constructors
  
  /**
   * Creates a new ClientBag
   */
  public ClientBag() {
  }  
  
  /**
   * Creates a new ClientBag knowing the number of letters that it must contain.
   */
  public ClientBag (int bagSize) {
    this.bagSize = bagSize;
  }  
  


  //Methods


  /**
   * Returns the number of letters left
   */
  public int getLetterNbr(){
    if (letters == null)
      return bagSize;
    
    return letters.size();
  }
  

  /**
   * 
   */
  void setLetterNbr(int newSize) {
    bagSize = newSize;
    updateLabel();
  }
  
  /**
   * 
   */  
  void updateLabel() {
    baglabel.setText(" Number of letters in the BAG :" + bagSize );
  }


  /**
   * Displays the letter that is containing
   */
  public String displayLetters(){
    return letters.toString(); 
  }

  /**
   * Invoked when a new data frame is received.
   * @param frame the frame that has just been received.
   */  
  public void frameReceived(Object link, LinkFrame frame) {
    int request = frame.getRequest();
    
    switch (request) {
      
    case RQ_BAGCHANGEBAGSIZE :
      setLetterNbr(((Integer) frame.getData()).intValue());
      break;
      
    case RQ_BAGCHANGELETTERS :
      letters = (Vector) frame.getData(); //@@ should use accessor -vm980406
      updateLabel();
      break;
      
    case RQ_DECREASE_LETTER_COUNT:
      if (debug)
	System.out.println("<ClientBag> frameReceived: RQ_DECREASE_LETTER_COUNT: decrease by " + frame.getSubRequest());
      setLetterNbr(getLetterNbr() - frame.getSubRequest());
      break; 
      
    default : 
      if (debug)
	System.out.println("<ClientBag> frameReceived Unhandled request");  
      break;
    }
  }

  
  /*
   * Returns a Label wich displays what the Bags is containing
   */  
  public Component getComponent(){
  
    if (baglabel == null) {
      baglabel = new JLabel ("Letters left in bag: " + getLetterNbr());
      baglabel.setBorder(new BevelBorder(BevelBorder.LOWERED));
      baglabel.setVerticalAlignment(JLabel.CENTER);
      baglabel.setHorizontalAlignment(JLabel.CENTER);
    }

    return baglabel;
  }  
  
}

