/*
 * $Id: ClientBoard.java,v 1.5 1998/04/07 12:31:21 vmallet Exp $
 *
 *
 * $Log: ClientBoard.java,v $
 * Revision 1.5  1998/04/07 12:31:21  vmallet
 * Added the frameReceived() method.
 *
 * Revision 1.4  1998/04/06 09:44:59  vmallet
 * Minor modification in getComponent().
 *
 * Revision 1.3  1998/04/05 20:16:53  lozac
 * Added the GUI getComponent() code.
 *
 * Revision 1.2  1998/04/05 16:49:35  vmallet
 * Removed the Displayable code.
 * added the getComponent() prototype.
 *
 * Revision 1.1  1998/04/03 16:33:46  poivert
 * Initial revision
 *
 *
 */

package scrobble;


import java.util.*;
import game.*;
import com.sun.java.swing.*;
import com.sun.java.swing.border.*;
import java.awt.*;


/**
 * ClientBoard.java
 * 
 * herite de Board et implemente l'interface Displayable
 *
 * @version $Revision: 1.5 $
 * @author  Frederique Poivert (poivert@enst.fr)
 */
public class ClientBoard extends Board implements GameLinkListener, ScrobbleConstants {

  /** 
   *  The panel wich represents the Board
   */
  JPanel boardPanel;

  /**
   * if true, enables the printing of debug informations on the console.
   */  
  protected boolean debug = true;


  // Constructor

  /**
   * Creates e new ClientBoardl
   */ 
  public ClientBoard(BoardShape shape) { 
    super(shape);
  }


  // Methods


  /**
   * Verifies that the given word can be put on the board or not. 
   * @param wordp the WordPositioner to be verified.
   * @return  true if the given word can be put on the board, and exception otherwise.
   */
  public boolean verify(WordPositioner wordp) throws IllegalPositionException {
    return tryAndFit(wordp) != null;
    
  }


  /**
   * Returns the GUI component representing this board.
   */  
  public Component getComponent() {    
    if (boardPanel == null) {
      boardPanel = new JPanel();
      boardPanel.setLayout(new GridLayout(shape.getWidth(), shape.getHeight()));
      boardPanel.setBorder(new MatteBorder(5, 5, 5, 5, Color.black));
      
      for (int i = 0 ; i < shape.getHeight() ; i++ ) {
	for (int j = 0 ; j < shape.getWidth() ; j++) {
	  boardPanel.add(squares[i][j].getComponent());
	}
      }
    }
    return boardPanel;
  }


  /**
   * Messaged when a new data frame is received by the <code>GameLinkFilter</code>
   * object and this frame belongs to the groups the GameLinkListener
   * asked to listen to.
   *
   * @param link  the GameLink that actually received the frame.
   * @param frame the frame that has just been received.
   *
   * @see GameLink
   * @see GameLinkListener#frameReceived
   */  
  public void frameReceived(Object link, LinkFrame frame) {
    int request = frame.getRequest();
    
    switch (request) {
    case BOARD_PUT_WORD:
      System.out.println("<ClientBoard> frameReceived(): BOARD_PUT_WORD");
      
      break;
      
    default:
      if (debug)
	System.out.println("<ClientBoard> frameReceived(): unhandled request: " + request);
      break;
    }
  }







  
} // class ClientBoard


 

