/*
 * $Id: ClientScrobble.java,v 1.4 1998/04/07 12:34:15 vmallet Exp $
 *
 *
 * $Log: ClientScrobble.java,v $
 * Revision 1.4  1998/04/07 12:34:15  vmallet
 * Handle the BOARD_CREATED request.
 * Cleaned up some code.
 *
 * Revision 1.3  1998/04/06 21:54:48  vmallet
 * Little more code.
 *
 * Revision 1.2  1998/04/06 09:56:00  vmallet
 * Lot of changes. First try at making this class work.
 *
 * Revision 1.1  1998/04/04 14:18:54  legras
 * Initial revision
 *
 *
 */

package scrobble;

import game.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import com.sun.java.swing.*;

/**
 * ClientScrobble.java
 * Is independant of Scrabble.
 * Its aim is to draw the Scrabble, a specific game.
 * It's the daugther of ClientGame.
 *
 * @version $Revision: 1.4 $
 * @author  Helene Legras (legras@enst.fr)
 */
public class ClientScrobble extends ClientGame implements GameLinkListener, ScrobbleConstants {

  /**
   * Board used by Scrobble
   */
  ClientBoard board;
  
  /**
   * Bag used by Scrobble
   */
  ClientBag bag;
  
  /**
   * Language used by Scrobble
   */
  ClientLanguage language;
  
  /**
   * Timer used by Scrobble
   */
  //@@Timer timer;
  

  
  // constructor

  /**
   * Creates a new ClientScrobble
   */
  public ClientScrobble() {
  } 
  

  //methods

  Container gamePanel;

  /**
   * 
   */
  public void make(GameLinkFilter link, int myPlayerId) {
    super.make(link, myPlayerId);
    gamePanel = createWindow(400, 450);   
    //    gamePanel.repaint();
    gamePanel.validate();
    
    //    new LetterPanel().sendWord(gameLink);
  }
  
  /**
   *
   */
  public Player createPlayer(int id, Object frameData){
    ClientScrobblePlayer player = new ClientScrobblePlayer(id, frameData);
    return player;
  }
 
  
  
  /**
   * 
   */
  public void frameReceived(Object link, LinkFrame frame) {
    int request = frame.getRequest();

    switch(request) {
    case BAG_CREATED:
      // a bag has been created
      Integer bagSize = (Integer) frame.getData(); 
      if (debug)
	System.out.println("<ClientScrobble> frameReceived BAG_CREATED: " + bagSize + " letters");
      bag = new ClientBag(bagSize.intValue());
      
      gameLink.addGameLinkListener(bag, BAG_GROUP);
      
      gamePanel.add(bag.getComponent(), BorderLayout.SOUTH);
      gamePanel.validate();
      
      break;
      
    case BOARD_CREATED:
      //a board has been created

      if (debug)
	System.out.println("<ClientScrobble> frameReceived BOARD_CREATED");
      
      board = new ClientBoard((BoardShape) frame.getData());

      gameLink.addGameLinkListener(board, BOARD_GROUP);

      gamePanel.add(board.getComponent(), BorderLayout.CENTER);
      gamePanel.validate();
      break;

    case LETTER_TAKEN:
      //a letter has been taken in the bag
      bag.getComponent();
      break;

    case WORD_FALSE:
      //the word is not accepted 
      if (frame.getSubRequest() == POSITION)
	System.out.println("Your word isn't accepted because of an illegal position. Please, try again.");
      else  System.out.println("Your word isn't accepted because it isn't in the dictionary. Please, try again.");
      break;

    case  BAG_EMPTY:
      //the bag is empty
      System.out.println("The bag is empty, we can't give you new letters.");
      break;

    case PLAYER_PLAYS:
      //the player is playing
      ClientScrobblePlayer player = (ClientScrobblePlayer) frame.getData();
      System.out.println("The player " + player.getName() + " is playing now");
      break;

    case  CONFIGURATION:
      //the object of configuration has to be writen
      doPromptConfig(frame);
      break;

    default:
      super.frameReceived(link, frame);
      break;
    }
  } // frameReceived



  /**
   * Upon reception of a configuration request, prompt user for game configuration.
   *
   * @param frame the frame containing the configuration request and data.
   */  
  public void doPromptConfig(LinkFrame frame) {
    LinkFrame answer = new LinkFrame();

    Configuration config = (Configuration) frame.getData();
    InitGamePanel gameInit = new InitGamePanel(config, gameLink);
  }

}
