
/*
 * $Id: Dico.java,v 1.2 1998/04/06 09:39:14 vmallet Exp $
 *
 * 
 * $Log: Dico.java,v $
 * Revision 1.2  1998/04/06 09:39:14  vmallet
 * Added "<Dico>" in all the console output.
 *
 * Revision 1.1  1998/04/05 15:29:50  dunand
 * Initial revision
 *
 *
 */
package scrobble;

import scrobble.square.*;

import java.io.*;
import java.util.*; 
import java.net.*;


/**
 * Dico.java
 *	
 * This class defines the <B>dictionnary</B> object. This object is 
 * used to Verify that a given word is in the database.  
 * 
 * @version $Revision: 1.2 $
 * @author  Loc Dunand (dunand@email.enst.fr)
 */

public class Dico { 

  /* Used to keep trace of loaded dictionnaries and the number of links on these Dicos */

class CountHash 
{
    int count;
    
    Hashtable hash;
    
    public CountHash(int count, Hashtable hash)
    {
      this.count=count;
      this.hash=hash;
    }
    
    public int getCount()
    {
      return(count);
    }
    
    public Hashtable getHash()
    {
      return(hash);
    }
    
    public void setCount(int count)
    {
      this.count=count;
    }
    
    public void setHash(Hashtable hash)
    {
      this.hash=hash;
    }
    
    
    public String toString()
    {
      return("Count for this hashtable : " + count );
    }
    
  }
  
  
  /**
   * if true, enables the printing of debug informations on the console.
   */  
  static public final boolean debug = true;
  
  /**
   * Contains the name of the file that will be used as a word source 
   */
  String dicoFile;

  /**
   * Contains all the words of the dictionnary created 
   */
  Hashtable listOfWords;
  Hashtable userWords;

  /**
   * Contains the average number of words in a dictionnary it is initialised to 50000
   */
  int averageLength = 50000;
  
  /**
   * Contains the list of all the Dictionnaries created 
   */
  static Hashtable DicoTable = new Hashtable();
  
  
  // Constuctors
  
  
  /**
   * Creates a new dictionnary with the given file. The following
   * Files can be used :
   *	"English.words" is English version with about 45000 words 
   *	"Little.words" is English version with about 25000 words 
   *    "French.words" is a test file for including 100 words
   *
   * @param dicoFile the identifier of language of the dictionnary being created
   */
  public Dico(String dicoFile) {
    listOfWords= new Hashtable(averageLength);  
    userWords = new Hashtable();
    this.dicoFile = dicoFile;
    
    if (debug)
      System.out.println("<Dico> dicoFile(): Loading list of words in Memory...");
    loadWords();
    if(debug)
      System.out.println("<Dico> dicoFile(): List of words loaded !");
  }
  
  
  // Methods
  
  
  /**
   * Checks if a given word is in the dictionnary
   * Returns a boolean </B>true</B> if the word is in the dictionnary and 
   * </B>false</B> if it isn't
   * @param toBeChecked the WordPositionnerthat has to be checked
   */
  public boolean isWord(WordPositioner toBeChecked) {
    return(isString(toBeChecked.getWord().wordToString()));
  }
  /**
   * Checks if a given string is in the dictionnary
   * Returns a boolean </B>true</B> if the string is in the dictionnary and 
   * </B>false</B> if it isn't
   * @param toBeChecked the string that has to be checked
   */
  public boolean isString(String toBeChecked) {	
    return(listOfWords.containsKey(toBeChecked)||userWords.containsKey(toBeChecked));
  }
  
  /**
   * Returns a string representation of the Dico.
   */
  public String toString ()
  {return(  dicoFile + " dictionnary contains :" + listOfWords.size() + " Elements  \n" +
	    "User Dico contains : " + userWords.size() + " Elements \n" +
	    "Number of dictionnaries linked to " + dicoFile + " : " + ((CountHash)DicoTable.get(dicoFile)).getCount() );
  
  }
  
  
  
  /**
   * Adds a given WordPositionner to the user dictionnary.
   * The modifications will only affect the dictionnary in
   * use and not the file <code>dicoFile</code>. 
   * @param toBeAdded the word that must be added to the dictionnary
   */
  public void addWord(WordPositioner toBeAdded) {
    String nullString =  "";
    userWords.put(toBeAdded.getWord().wordToString(),nullString);
  }
  
  
  /**
   * <I>Optionnal</I> : Gives the definition of a given word
   * Returns a String object containing the words definition 
   * @param toBeDefined the word that must be defined
   */
  public String giveDefinition(WordPositioner toBeDefined) {
    return("This Function is not yet implemented !");
    
  }
  
  void loadWords() {
    int count;
    CountHash newCountHash;
    synchronized(DicoTable) {
      if (DicoTable.containsKey(dicoFile)) {
	listOfWords=((CountHash)DicoTable.get(dicoFile)).getHash();
	count=((CountHash)DicoTable.get(dicoFile)).getCount()+1;
	((CountHash)DicoTable.get(dicoFile)).setCount(count);
      }
      else 
	{
	  loadFile();
	  newCountHash = new CountHash(1,listOfWords);
	  DicoTable.put(dicoFile,newCountHash);
	}
    }
  }
  
  /**
  * This method is used to remove a link on the current dictionnary 
  */
  public void unloadWords() {
    int count;
    synchronized(DicoTable) {
      count=((CountHash)DicoTable.get(dicoFile)).getCount()-1;
      if (count>0) ((CountHash)DicoTable.get(dicoFile)).setCount(count);
      else ((CountHash)DicoTable.get(dicoFile)).setHash(null);
    }
    
  }
  
  /**
  * This methode uses the <code>unloadWords()<\code> methode when the garbage collector deletes this object.
  */
  

  protected void finalize() {
    unloadWords();
    
  }
  
  
  /**
   * This method loads the words in the <code>dicoFile<\code> and adds them to the
   * <code>listOfWords</code> Vector. This function has no parameters.
   */
  
  void loadFile() {
    URL               url        = null;
    InputStream       sourceFlow = null;
    InputStreamReader reader;
    BufferedReader    formatedSource;
    String            line;
    
    try
      {
	// url = new URL("file","",dicoFile);
	url            = getClass().getResource(dicoFile);
	sourceFlow     = url.openStream();

	if (debug) 
	  System.out.println("<Dico> loadFile(): url=" + url);

	reader         = new InputStreamReader(sourceFlow);
	formatedSource = new BufferedReader(reader);
	//listOfWords.clear();
	/* Until the end of the file has not been reached the word non the current line 
	   of the file is added to the listOfWords vector */
	while ((line=formatedSource.readLine())!=null) 
	  listOfWords.put(line,"");
      }
    catch (FileNotFoundException e)
      {
	if (debug)
	  System.out.println("<Dico> loadFile(): File not found: ");
      }
    catch (IOException e)
      {
	if (debug)
	  System.out.println("<Dico> loadFile(): Read Error");
      }
    finally
      {
	try
	  {
	    sourceFlow.close();
	  }
	catch(IOException e)
	  {
	    if (debug)
	      System.out.println("<Dico> loadFile(): Closing Error while reading");
	  }
      }
  }  
} 









