/*
 * $Id: Letter.java,v 1.5 1998/04/06 19:56:31 vmallet Exp $
 *
 *
 * $Log: Letter.java,v $
 * Revision 1.5  1998/04/06 19:56:31  vmallet
 * Revisited the getComponent() method.
 * Reduced the letter component size.
 *
 * Revision 1.4  1998/04/06 18:51:24  vmallet
 * Now implements Serializable.
 * Reduced the letter component size.
 *
 * Revision 1.3  1998/04/06 09:43:00  vmallet
 * Modified the getComponent() method. Should work better.
 *
 * Revision 1.2  1998/04/05 20:15:22  lozac
 * Added the GUI getComponent() code.
 *
 * Revision 1.1  1998/04/04 14:18:26  legras
 * Initial revision
 *
 *
 */

package scrobble;

import game.*;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import com.sun.java.swing.*;
import com.sun.java.swing.border.*;


/**
 * Letter is able to give its name and its value.
 * 
 * @version $Revision: 1.5 $
 * @author  Helene Legras (legras@enst.fr)
 *
 */
public class Letter implements Serializable {

  /**
   *Name of Letter
   */
  char name;

  /**
   *Value of Letter
   */
  int value;

  /**
   * The label which contains the name of the letter;
   */
  protected JLabel nameLabel;
  
  /**
   * The label which contains the value of the letter;
   */
  protected JLabel valueLabel;


   /**
   * The panel which represents the letter;
   */
  protected JPanel letterPanel;

  //constructors

  /**
   * Creates a new Letter with the given name
   * and the value 1 if nothing is given.
   * @param name the name of the new Letter being created
   */
  public Letter(char name){
    this(name,1);
  }

  /**
   * Creates a new Letter with the given name
   * and the given value.
   * @param name the name of the new Letter being created
   * @param value the value of the new letter being created
   */
  public Letter(char name, int value){
    this.name=name;
    this.value=value;
  }

  /**
   * Creates a new Letter without any parameter
   */
  public Letter(){
  }


 //methods
  
  /**
   * Changes the name of the Letter
   * @param newname the new name of the Letter
   */
  public void setName(char newname){
    name = newname;
  }

  /**
   *Returns the name of the Letter
   *@return the name
   */
  public char getName() {
    return name;
  }

  /**
   * Changes the value of the Letter
   * @param newvalue the new value of the Letter
   */
  public void setValue(int newvalue){
   value = newvalue;
  }

  /**
   *Returns the value of the Letter
   *@return the value
   */
  public int getValue() {
    return value;
  }

  /**
   * Tests if Letter has the same name and the same value
   * that the Letter in parameter has.
   * @param letter the other letter.
   * @return true or false
   */
  public boolean equals(Letter letter){
    if (name == letter.getName() && value == letter.getValue()){
      return true;
    }
    else{
      return false;
    }
  }
  
  /**
   * Returns a string representation of the object.
   */
  public String toString() {  
    return "(" + getName() + ", " + getValue() + ")";
  }


  /**
   * Returns a label containing the letter (his name and his value).
   */  
  public Component getComponent() {
    if (letterPanel == null) {
      Dimension dim = new Dimension(19, 19);
      
      letterPanel = new JPanel();
      letterPanel.setMinimumSize(dim);
      letterPanel.setPreferredSize(dim);
      letterPanel.setMaximumSize(dim);
      letterPanel.setBackground(SystemColor.control);
      letterPanel.setLayout(new BoxLayout(letterPanel, BoxLayout.X_AXIS));
      letterPanel.setBorder(new BevelBorder(BevelBorder.RAISED));
      
      Font nameFont  = new Font("Serif", Font.BOLD, 12);
	
      if (getName() != 0) {
	Font valueFont = new Font("Serif", 0, 7);

	nameLabel = new JLabel("" + getName());
	nameLabel.setFont(nameFont);
	valueLabel = new JLabel("" + getValue());
	valueLabel.setFont(valueFont);

	letterPanel.add(nameLabel);
	letterPanel.add(valueLabel);

      }
      else {
	nameLabel = new JLabel("?");
	nameLabel.setFont(nameFont);

	letterPanel.add(nameLabel);
      }
    }
    
    return letterPanel;
  }
} 
