/*
 * $Id: Scrobble.java,v 1.8 1998/04/07 12:35:23 vmallet Exp $
 *
 *
 * $Log: Scrobble.java,v $
 * Revision 1.8  1998/04/07 12:35:23  vmallet
 * Rewrote removePlayer().
 * Handling of the ST_PLAY_WORD request.
 *
 * Revision 1.7  1998/04/06 21:56:07  vmallet
 * Some more code.
 *
 * Revision 1.6  1998/04/06 11:26:39  lozac
 * Second attempt to create a correct BoardShape
 *
 * Revision 1.5  1998/04/06 10:55:35  vmallet
 * First attempt to create a correct BoardShape.
 *
 * Revision 1.4  1998/04/06 09:56:25  vmallet
 * Lot of changes. First try at making this class work.
 *
 * Revision 1.3  1998/04/04 17:52:07  vmallet
 * Changed addNewPlayer to addPlayer
 *
 * Revision 1.2  1998/04/04 14:06:15  legras
 * Changement de ScrobbleLinkConstants en ScrobbleConstants.
 *
 * Revision 1.1  1998/04/04 13:57:09  legras
 * Initial revision
 *
 *
 */
package scrobble;

import game.*;
import scrobble.square.*;

import java.util.*;


/**
 * Scrobble.java
 * Implement the different methods mentionned in Game class.
 *
 * @version $Revision: 1.8 $
 * @author  Helene Legras (legras@enst.fr)
 */
public class Scrobble extends Game implements GameLinkConstants, ScrobbleConstants {
  
  
  /**
   * Board used by Scrabble
   */
  ServerBoard board;
  
  /**
   * Bag used by Scrabble
   */
  ServerBag bag;
  
  /**
   * Dictionnary used by Scrabble
   */
  Dico  dictionnary;
  
  /**
   * Language used by Scrabble
   */
  ServerLanguage language;
  
  /**
   * Hourglass used by Scrabble
   */
  Timer timer;
  
  Observable observable;

  String nameLanguage; 
  BoardShape boardshape;
  int max;
  boolean privateGame;

  /**
   * if true, enables the printing of debug informations on the console.
   */  
  protected boolean debug = true;


  //methods

  /**
   * Creats a new Player
   * @param name the name of the new player
   * @param gamelink the link of the player with the net 
   * @return the new player 
   */
  public Player createPlayer(String name, GameLinkFilter gamelink){   
    ServerScrobblePlayer player = new ServerScrobblePlayer(name, gamelink); 
    
    if (componentsCreated) {
      Vector letters = null;
      try {
	letters = bag.getLetters(7); //@@ a l'arrache
      } catch (Exception e) {
	System.out.println("<Scrobble> createPlayer(): bag.getLetters() exception: " + e);
	e.printStackTrace();
	//@@ do sth
      }
      player.addLetters(letters); //@@ should bug as of today 980406
    }

    return player;
  }

  // commented 98.04.05 -vim
//   /**
//    * Adds a Player to the Game
//    * Exeptions are made if it isn't possible
//    * @param player the player who has to be addicted
//    */
//   public void addPlayer(ServerScrobblePlayer player) throws TooManyPlayersException, ClosedGameException {
//    super.addPlayer(player);

  /**
   * 
   */  
  public void setupPlayer(Player player) {
    super.setupPlayer(player);
    if (debug)
      System.out.println("<Scrobble> setupPlayer(): " + player);

    //@@ check if there can be some race problem here.
    
    if (getPlayerCount() == 1) {   // first player gets configuration request.
      if (debug)
	System.out.println("   +sending config object");

      // Make Configuration
      Configuration config = new Configuration();

      config.setBoardShape(createScrobbleShape());

      Enumeration languages = ServerLanguage.getLanguages();
      config.setLanguages(languages);

      LinkFrame frame = new LinkFrame();
      frame.setGroup(GAME_GROUP);
      frame.setRequest(CONFIGURATION, NONE);
      frame.setData(config);
      player.getGameLink().sendBlindly(frame);
    }

    updatePlayer(player);
  }


  /**   */  
  boolean componentsCreated = false;
  
  /**
   * 
   */  
  public void createGameComponents(Configuration config) {
    if (debug)
      System.out.println("<Scrobble> createGameComponents()");
    if (componentsCreated)  // prevent from possible client bug.
      return;
    
    configure(config);

    language = new ServerLanguage(config.getSelectedLanguageId());
    
    bag = new ServerBag(language.getDistri());

    dictionnary = language.getDico();
    
    board = new ServerBoard(boardshape);

    componentsCreated = true;

    for (Enumeration e = getPlayers(); e.hasMoreElements(); ) {
      Player p = (Player) e.nextElement();
      updatePlayer(p);
    }
  }

  /**
   * [DESCRIBE ME]
   */  
  public void updatePlayer(Player player) {
    if (debug)
      System.out.println("<Scrobble> updatePlayer(): " + player);
    
    // can't update player if components not created.
    if (!componentsCreated)
      return;
    
    if (debug)
      System.out.println("    + not bypassed");

    GameLinkFilter gameLink = player.getGameLink();
    // sends order to create bag, board, players in new ClientGame
    LinkFrame frame = new LinkFrame();
    frame.setGroup(GAME_GROUP);
    
    frame.setRequest(BAG_CREATED, NONE);
    frame.setData(new Integer(bag.getSize()));
    gameLink.sendBlindly(frame);

    bag.addObserver(player);
    
    ServerScrobblePlayer p = (ServerScrobblePlayer) player;
    
    if (p.getNbLetters() == 0) {
      Vector letters = null;
      try {
	letters = bag.getLetters(7); //@@ a l'arrache
      } catch (Exception e) {
	System.out.println("<Scrobble> updatePlayer(): bag.getLetters() exception: " + e);
	//	e.printStackTrace();
	//@@ do sth
      }
      p.addLetters(letters); //@@ should bug as of today 980406
    }
    else {
      p.addLetters(new Vector()); //@@ HACK ! hahahahahah
    }

    frame.setRequest(BOARD_CREATED, NONE);
    frame.setData(boardshape);
    gameLink.sendBlindly(frame);

//     for (int i = 1 ; i <= players.size();i++){
//       ServerScrobblePlayer player1 = (ServerScrobblePlayer)players.elementAt(i);
//       frame.setRequest(PLAYER_CREATED, NONE);
//       Vector nameAndScore = new Vector();
//       Integer score = new Integer(player1.getScore());
//       nameAndScore.addElement(player1.getName());
//       nameAndScore.addElement(score);
//       frame.setData(nameAndScore);
//       GameLinkFilter gameLinkOthers = player1.getGameLink();
//       gameLinkOthers.sendBlindly(frame);
//     }
    
    
//     // says to other players that an other one is coming
//     frame.setRequest(PLAYER_CREATED, NONE);
//     Vector nameAndScore = new Vector();
//     Integer score = new Integer(player.getScore());
//     nameAndScore.addElement(player.getName());
//     nameAndScore.addElement(score);
//     frame.setData((Object)nameAndScore);
//     notifyObservers(frame);
    // now he is an observer of all other objects
    board.addObserver(player);
//     for (int i = 1 ; i<= players.size(); i++){
//       ServerScrobblePlayer player1 = (ServerScrobblePlayer)players.elementAt(i);
//       player1.addObserver(player);
//     }
  }
  

  /**
   * Removes a Player from the Game
   * @param player the player who will be removed
   */
  public void removePlayer(Player player){
    if (debug)
      System.out.println("<Scrobble> removePlayer(): " + player) ;
    
    bag.deleteObserver(player);
    board.deleteObserver(player);

    super.removePlayer(player);

    // put letters from player back into the bag
    Vector letters = new Vector();
    for (Enumeration e = ((ServerScrobblePlayer) player).getLetters(); e.hasMoreElements(); ) {
      letters.addElement(e.nextElement());
    }
    bag.addLetters(letters);



//     Enumeration lettersPlayer = player.getLetters();
//     while (lettersPlayer.hasMoreElements()){
//       bag.setLetter((Letter)lettersPlayer.nextElement());
//     }
//     player.removeLetters();
  }


  /**
   * Gives the configuration that the first player wants
   * @ param the configuration object  
   */
  public void configure(Configuration config){
    nameLanguage = config.getLanguage();
    boardshape = config.getBoardShape();
    max = config.getMaxPlayer();
    privateGame = config.isPrivate();
  }

  /**
   * Returns the BoardShape that will make the board look like
   * a scrabble board.
   */  
  public BoardShape createScrobbleShape() {

    //@@ config du shape a la main... devrait pas ! -vim980406
    Square                   sq            = new ServerTripleWordSquare();
    SquarePositioner         sp            = new SquarePositioner(0, 0, sq);  
    Vector                   squareList    = new Vector();
    squareList.addElement(sp);

    sq = new ServerTripleWordSquare();
    sp = new SquarePositioner(0, 7, sq);
    squareList.addElement(sp);
    sq = new ServerTripleWordSquare();
    sp = new SquarePositioner(0, 14, sq);
    squareList.addElement(sp);
    sq = new ServerTripleWordSquare();
    sp = new SquarePositioner(7, 0, sq);
    squareList.addElement(sp);
    sq = new ServerTripleWordSquare();
    sp = new SquarePositioner(14, 0, sq);
    squareList.addElement(sp);
    sq = new ServerTripleWordSquare();
    sp = new SquarePositioner(7, 7, sq);
    squareList.addElement(sp);
    sq = new ServerTripleWordSquare();
    sp = new SquarePositioner(7, 14, sq);
    squareList.addElement(sp);
    sq = new ServerTripleWordSquare();
    sp = new SquarePositioner(14, 7, sq);
    squareList.addElement(sp);
    sq = new ServerTripleWordSquare();
    sp = new SquarePositioner(14, 14, sq);
    squareList.addElement(sp);

    sq = new ServerDoubleWordSquare();
    sp = new SquarePositioner(1, 1, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleWordSquare();
    sp = new SquarePositioner(2, 2, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleWordSquare();
    sp = new SquarePositioner(3, 3, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleWordSquare();
    sp = new SquarePositioner(4, 4, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleWordSquare();
    sp = new SquarePositioner(1, 13, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleWordSquare();
    sp = new SquarePositioner(2, 12, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleWordSquare();
    sp = new SquarePositioner(3, 11, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleWordSquare();
    sp = new SquarePositioner(4, 10, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleWordSquare();
    sp = new SquarePositioner(13, 1, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleWordSquare();
    sp = new SquarePositioner(12, 2, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleWordSquare();
    sp = new SquarePositioner(11, 3, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleWordSquare();
    sp = new SquarePositioner(10, 4, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleWordSquare();
    sp = new SquarePositioner(13, 13, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleWordSquare();
    sp = new SquarePositioner(12, 12, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleWordSquare();
    sp = new SquarePositioner(11, 11, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleWordSquare();
    sp = new SquarePositioner(10, 10, sq);
    squareList.addElement(sp);

    sq = new ServerDoubleLetterSquare();
    sp = new SquarePositioner(0, 3, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleLetterSquare();
    sp = new SquarePositioner(0, 11, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleLetterSquare();
    sp = new SquarePositioner(3, 0, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleLetterSquare();
    sp = new SquarePositioner(11, 0, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleLetterSquare();
    sp = new SquarePositioner(14, 3, sq);
    squareList.addElement(sp);      
    sq = new ServerDoubleLetterSquare();
    sp = new SquarePositioner(14, 11, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleLetterSquare();
    sp = new SquarePositioner(3, 14, sq);
    squareList.addElement(sp);      
    sq = new ServerDoubleLetterSquare();
    sp = new SquarePositioner(11, 14, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleLetterSquare();
    sp = new SquarePositioner(6, 6, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleLetterSquare();
    sp = new SquarePositioner(6, 8, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleLetterSquare();
    sp = new SquarePositioner(8, 6, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleLetterSquare();
    sp = new SquarePositioner(8, 8, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleLetterSquare();
    sp = new SquarePositioner(2, 6, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleLetterSquare();
    sp = new SquarePositioner(2, 8, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleLetterSquare();
    sp = new SquarePositioner(3, 7, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleLetterSquare();
    sp = new SquarePositioner(6, 2, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleLetterSquare();
    sp = new SquarePositioner(8, 2, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleLetterSquare();
    sp = new SquarePositioner(7, 3, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleLetterSquare();
    sp = new SquarePositioner(6, 12, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleLetterSquare();
    sp = new SquarePositioner(8, 12, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleLetterSquare();
    sp = new SquarePositioner(7, 11, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleLetterSquare();
    sp = new SquarePositioner(12, 6, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleLetterSquare();
    sp = new SquarePositioner(12, 8, sq);
    squareList.addElement(sp);
    sq = new ServerDoubleLetterSquare();
    sp = new SquarePositioner(11, 7, sq);
    squareList.addElement(sp);

    sq = new ServerTripleLetterSquare();
    sp = new SquarePositioner(1, 5, sq);
    squareList.addElement(sp);
    sq = new ServerTripleLetterSquare();
    sp = new SquarePositioner(1, 9, sq);
    squareList.addElement(sp);
    sq = new ServerTripleLetterSquare();
    sp = new SquarePositioner(5, 1, sq);
    squareList.addElement(sp);
    sq = new ServerTripleLetterSquare();
    sp = new SquarePositioner(9, 1, sq);
    squareList.addElement(sp);
    sq = new ServerTripleLetterSquare();
    sp = new SquarePositioner(5, 13, sq);
    squareList.addElement(sp);
    sq = new ServerTripleLetterSquare();
    sp = new SquarePositioner(9, 13, sq);
    squareList.addElement(sp);
    sq = new ServerTripleLetterSquare();
    sp = new SquarePositioner(13, 5, sq);
    squareList.addElement(sp);
    sq = new ServerTripleLetterSquare();
    sp = new SquarePositioner(13, 9, sq);
    squareList.addElement(sp);    
    sq = new ServerTripleLetterSquare();
    sp = new SquarePositioner(5, 5, sq);
    squareList.addElement(sp);
    sq = new ServerTripleLetterSquare();
    sp = new SquarePositioner(5, 9, sq);
    squareList.addElement(sp);
    sq = new ServerTripleLetterSquare();
    sp = new SquarePositioner(9, 5, sq);
    squareList.addElement(sp);
    sq = new ServerTripleLetterSquare();
    sp = new SquarePositioner(9, 9, sq);
    squareList.addElement(sp);

    ServerSquare defaultSquare = new ServerSquare();
    BoardShape   shape         = new BoardShape(15, 15, defaultSquare, squareList);
    
    return shape;
  }







    /**
   * Messaged when a new data frame is received by the <code>GameLinkFilter</code>
   * object and this frame belongs to the groups the GameLinkListener
   * asked to listen to.
   *
   * @param link  the GameLink that actually received the frame.
   * @param frame the frame that has just been received.
   *
   * @see GameLink
   * @see GameLinkListener#frameReceived
   */  
  public void frameReceived(Object link, LinkFrame frame) {
    int request = frame.getRequest();

    switch (request) {
    case CONFIGURATION_CHOICE_MADE:{
      Configuration config = (Configuration) frame.getData();
      createGameComponents(config);
      break;
    }

    case ST_PLAY_WORD:
      if (debug)
	System.out.println("<Scrobble> frameReceived: ST_PLAY_WORD");
      
      WordPositioner wordp = (WordPositioner) frame.getData();
      
      int score = 0;
      try {
	score = board.verifyAndCount(wordp, dictionnary);
	System.out.println("<Scrobble> ST_PLAY_WORD: word passed !");
	board.putWord(wordp);
      } catch (Exception e) {
	System.out.println("<Scrobble> ST_PLAY_WORD: word failed: " + wordp + "  exception: " + e );
      }
      
      break;

    default:
      super.frameReceived(link, frame);
      break;
    }
  }

  
    
  /**
   * Makes Player play a turn
   * @param player the new player
   */
  public void yourTurn(ServerScrobblePlayer player) {
    // tells to other players that player is playing
    LinkFrame frame = new LinkFrame();
    //frame.setGroup(GAME_GROUP);
    //frame.setRequest(PLAYER_PLAYS, NONE);
    //frame.setData((Object)player);
    //notifyObservers(frame);

    //timer?
   
    WordPositioner wordp = null;//@@comment player me donne son mot?
    int score = 0;
    //@@wordp = player.getWord();
    try {
	 score = board.verifyAndCount(wordp, dictionnary);
    }

    catch (IllegalPositionException e1) {
       GameLinkFilter gameLink = player.getGameLink();
       //LinkFrame frame = new LinkFrame();
       frame.setGroup(GAME_GROUP);
       frame.setRequest(WORD_FALSE, POSITION);
       gameLink.sendBlindly(frame);
      //word not allowed 
    }

    catch (IllegalWordException e2){
       GameLinkFilter gameLink = player.getGameLink();
       //LinkFrame frame = new LinkFrame();
       frame.setGroup(GAME_GROUP);
       frame.setRequest(WORD_FALSE, WORD);
       gameLink.sendBlindly(frame);
       //word not allowed
    }

    board.putWord(wordp);
    player.setScore(player.getScore() + score); 
    int nbPlayerLetters = player.getNbLetters();   
    for (int i = nbPlayerLetters ; i <= 7; i++) {

      //peut-etre entrer le nb de lettres de chaque joueur ds properties j'ai mis 7 par defaut
      Letter letter = new Letter();
      try {
	letter = bag.getLetter();
      }

      catch (Exception e1){  //exception si sac vide
        //LinkFrame frame = new LinkFrame();
	frame.setGroup(GAME_GROUP);
	frame.setRequest(BAG_EMPTY, NONE);
	notifyObservers(frame);
      }

      player.addLetter(letter);
      //LinkFrame frame = new LinkFrame();
      frame.setGroup(GAME_GROUP);
      frame.setRequest(LETTER_TAKEN, NONE);
      notifyObservers(frame); 
    }
  }

  /**
   * Plays a party of scrobble
   */
  public void start() {
    while(players.size()>1){
      for (int i = 1; i <= players.size(); i++) {
	yourTurn((ServerScrobblePlayer) players.elementAt(i));
      }
    }
  }
}




  



















































