/*
 * $Id: ScrobblePlayer.java,v 1.7 1998/04/07 12:29:38 vmallet Exp $
 * 
 *
 * $Log: ScrobblePlayer.java,v $
 * Revision 1.7  1998/04/07 12:29:38  vmallet
 * Added the toString() method.
 *
 * Revision 1.6  1998/04/06 21:53:53  vmallet
 * Fixed a bug in ScrobblePlayer(id, frameData).
 *
 * Revision 1.5  1998/04/06 18:53:33  vmallet
 * Changed the ScrobblePlayer(id, frameData) constructor.
 *
 * Revision 1.4  1998/04/05 20:03:56  lozac
 * Added getFrameData() code.
 * Better handling of letters transmission to client.
 *
 * Revision 1.3  1998/04/03 19:36:34  vmallet
 * Added a the ScrobblePlayer(name, id) constructor.
 *
 * Revision 1.2  1998/04/03 18:34:35  lozac
 * *** empty log message ***
 *
 * Revision 1.1  1998/04/03 18:12:20  lozac
 * Initial revision
 *
 */
 
package scrobble;

import game.*;
import java.io.*;
import java.util.*; 
import java.net.*; 
import java.awt.*;
import java.awt.event.*;
import com.sun.java.swing.*;

/**
 * The <code>ScrobblePlayer</code> class adds the two methods frameReceived 
 * and createLinkframe to <code>Player</code>, so that a  
 * <code>ScrobblePlayer</code> can create or receive a frame. It implements 
 * interface <code>GameLinkListener</code>. 
 *
 * @author  Kristell Lozac'hmeur(lozac@email.enst.fr)
 * @version $Revision: 1.7 $
 */
public class ScrobblePlayer extends Player {

  /**
   * Contains the Letters of Player.
   */
  Vector playletters; 

  /**
   * Number of Letters of Player
   */
  int nbletters;
  

  /**
   * The request which indicates the changement of letters of Player.
   */
  public static final int RQ_PLAYERCHANGELETTER = 6010;

  /**
   * The request which changes the changement of the number of letters of Player.
   */
  public static final int RQ_PLAYERCHANGENBLETTERS = 6020;


  // Constructor

  /**
   * Creates a new ScrobblePlayer
   */
  public ScrobblePlayer() { 
    super();
  }  

    /**
     * Creates a new ScrobblePlayer with his name
     * @param name the ScrobblePlayer's name.
     */
    public ScrobblePlayer(String name) { 
      super(name);
    }

  /**
   * Creates a new ScrobblePlayer with the informations contained in a 
   * object. 
   * @param frameData the object which contains the informations about the ScrobblePlayer.
   */
  public ScrobblePlayer(int id, Object frameData) {
    super(id, ((Object[]) frameData)[0]);

    Object[] frameArray = (Object[]) frameData;

    if (((Integer) ((Object[]) frameArray[0])[0]).intValue() == OTHER_DATA) {
      nbletters = ((Integer) frameArray[1]).intValue();
    }
    else {
      playletters = (Vector) frameArray[1];
    }

    System.out.println("<Player> init(id, frameData): " + this);

  }

  /**
   * Creates a new ScrobblePlayer with his name and a gamelink
   * @param name the ScrobblePlayer's name.
   * @param gamelink the ScrobblePlayer's gamelink.
   */
  public ScrobblePlayer(String name, GameLinkFilter gamelink) {  
    super(name, gamelink);
  }


  /**
   * Creates a new ScrobblePlayer with his name and his identity.
   *
   * @param name the ScrobblePlayer's name.
   * @param id the identity of Player.
   */
  public ScrobblePlayer(String name, int id) {
    super(name, id);
  }

  // Methods

  /**
   * adds a vector of Letters in the letters of Player and increases the number 
   * of Letters of the Player by the number of Letters added.  
   * @param vector the Letter to be added.
   */
  public void addLetters(Vector vector) {
    LinkFrame frame;
    System.out.println("<ScrobblePlayer> addLetters(): letters=" + vector);
    System.out.println("<ScrobblePlayer> addLetters(): playletters=" + playletters);
    for (int i = 0 ; i < vector.size(); i++) {
      playletters.addElement((Letter) vector.elementAt(i));  
    }
    System.out.println("<ScrobblePlayer> addLetters(): playletters=" + playletters);
    
  }

  /**
   * Returns the number of Player's Letters.
   * @return the number of Letters of the Player.
   */
  public int getNbLetters() {
    if (playletters == null) return nbletters;
    else return playletters.size();
  }

    /**
   * Returns the Letters of Player.
   * @return an enumeration of Player's Letters.
   */
  public Enumeration getLetters() {
    return playletters.elements();
  }

  /**
   * Returns the Letters of Player at the specified index.
   * @return the vector of Player's Letters.
   * @param index the index of the letter returned.
   */
  public Letter getLetterAt(int index) {
    return ((Letter) playletters.elementAt(index));
  }
  
 /**
   * Returns an object which contains the informations necessaries for the 
   * constructions by a new client of a Scrobbleplayer, which still exists in the server.
   * @return the object which contains the informations about the scrobbleplayer.
   */  
  public Object getFrameData() { 
    Object[] frameData = new Object[2];
    
    frameData[0] = super.getFrameData();
    frameData[1] = new Integer(getNbLetters());

    return frameData;
  }

 /**
   * Returns an object which contains the informations necessaries for the 
   * constructions by a new client of its own scrobbleplayer.
   * @return the object which contains the informations about the scrobbleplayer.
   */  
  public Object getFrameDataPerso() {
    Object[] frameData = new Object[10];
    Vector letterVector = new Vector();
    
    frameData[0] = super.getFrameDataPerso();
    for (Enumeration e = getLetters() ; e.hasMoreElements() ; ) {
      letterVector.addElement(e.nextElement());
      }
    frameData[1] = letterVector;

    return frameData;
  }

  /**
   * Returns a string representation of the object.
   */
  public String toString() {
    return super.toString() + " " + nbletters + ": " + playletters;
  }
}

