/*
 * $Id: ServerBag.java,v 1.5 1998/04/07 12:32:10 vmallet Exp $
 *
 *
 * $Log: ServerBag.java,v $
 * Revision 1.5  1998/04/07 12:32:10  vmallet
 * Rewrote the setNewLetters() method.
 * Renamed setNewLetters() to addLetters().
 *
 * Revision 1.4  1998/04/06 17:31:42  vmallet
 * Rewrote getLetter() and getLetters() that did not work.
 * Fixed bugs in various methods.
 * Other small changes.
 *
 * Revision 1.3  1998/04/05 17:46:28  granier
 * Insertion of debug labels
 *
 * Revision 1.2  1998/04/05 12:51:15  granier
 * If cheat, we transmit the whole vector Letters through the net.
 *
 * Revision 1.1  1998/04/05 11:40:35  granier
 * Initial revision
 *
 *
 */

package scrobble;
import java.util.*;
import java.lang.*;
import game.*;

/**
 * This class is the Server version of the class <code>Bag</code>.
 * It handles all the Bag operations (giving and receiving letter...) 
 * @version $Revision: 1.5 $
 * @author  Anne Granier (granier@enst.fr)
 */

public class ServerBag extends Bag implements GameLinkConstants{
  
  /**
   * if true, enables the printing of debug informations on the console.
   */  
  static boolean debug = true;

 
  /**
   * ServerBag  must be Observable by the others ClientBag. 
   * So needs the object o to have the methods addObserver(), deleteObserver() 
   * and notifyObserver().
   */
  
  protected Observable o;
  
  /** 
   *Boolean cheat enable to play in a cheaty way.
   */

  boolean cheat;

  //Constructor
  
  /**
   * Creates a new ServerBag
   */
  public ServerBag(Distribution distribution) {
    super(distribution);
    o = new Observable() {
      public void notifyObservers(Object o) {
	setChanged();
	super.notifyObservers(o);
      }
    };
  }
  
  /**
   * Creates a new ServerBag
   */
  public ServerBag() {  
    o = new Observable() {
      public void notifyObservers(Object o) {
	setChanged();
	super.notifyObservers(o);
      }
    };  
  }

    //Methods
  
  /********************************************************************************
										   
   * This paragraph concerns Obsevables: the Clientbag must be able to receive
   * a frame from BagServer through the player.
   
   */
  
  
  /**
   * Adds an observer to the set of observers for this object.
   * @param obs an observer to be added.
   */
  public void addObserver(Observer obs) {
    o.addObserver(obs);
  }
  
  /**
   * Deletes an observer to the set of observers for this object.
   * @param obs the observer to be deleted
   */
  public void deleteObserver(Observer obs) {
    o.deleteObserver(obs);
  }
  
  /**
   * Clears the observerlist so that this object no longer has any observers.
   */
  public void deleteObservers() {
    o.deleteObservers();
  }
  
  /**
   * Returns the number of observers of this object.
   * @return the number of observers of this object.
   */  
  public int countObservers() {
    return o.countObservers();
  }

  /**
   * [Describe me]
   */
  public void notifyObservers(LinkFrame frame) {
    System.out.println("<ServerBag> notifyObservers(): obs=" + countObservers());
    o.notifyObservers(frame);
  }

  // == End of observable part =====================================
  
  

  /**
   * Creates a new LinkFrame to tranmit to ClientBag. 
   * @param request the request of the frame.
   * @param data the data of the frame.
   */  
  public LinkFrame createLinkframe(int request, Object data) {
    LinkFrame frame = new LinkFrame();
    
    frame.setGroup(BAG_GROUP);
    frame.setRequest(request, NONE);
    frame.setData(data);
    
    return frame;  
  }
  
  /**
  * Set the modification happened to the bag, 
  * notify the observer.
  * @param  newLetters the new vector of letters remaining in the Bag.
  */
  public void addLetters(Vector newLetters) {
    for (Enumeration e = newLetters.elements(); e.hasMoreElements(); ) {
      letters.addElement(e.nextElement());
    }

    LinkFrame frame;

    if (cheat) 
      frame = createLinkframe(RQ_BAGCHANGELETTERS, letters);
    else
      frame = createLinkframe(RQ_BAGCHANGEBAGSIZE, new Integer(letters.size()));

    notifyObservers(frame);
  }

  

  /**
   * helper func. Exception has to be created properly (ie EmptyBagException or ..) -vim980406
   */  
  Letter getRandomLetter() throws Exception {
    int    idx    = (int) (letters.size() * Math.random());
    Letter letter = null;
    try {
      letter = (Letter) letters.elementAt(idx);
      letters.removeElementAt(idx);
    }
    catch (ArrayIndexOutOfBoundsException e) {
      throw new Exception("Empty bag: idx=" + idx);
    } 
    
    return letter;
  }
  
  
  /**
   * Get a letter
   * The letters in the Bag are organized
   * following  a Vector structure, built with a random process.
   * Eventually, the Bag returns a random letter. 
   * Faire une exception si le sac est vide
   */
  public Letter getLetter() throws Exception {
    Letter letter = getRandomLetter();
    
    // now notify observers that we lost one letter.
    LinkFrame frame = new LinkFrame();
    frame.setGroup(BAG_GROUP);
    frame.setRequest(RQ_DECREASE_LETTER_COUNT, 1); // 1 = 1 letter
    
    notifyObservers(frame);

    return letter;
  }

  /**
   * Removes one or more letters from the bag and return them.
   *
   * @param n the number of letters to remove from the bag
   * @return the letters actually removed from the bag
   * @exception Exception @@ [PENDING: Create me, describe me]
   */  

  public Vector getLetters(int n) throws Exception {
    Vector removedLetters = new Vector();
    
    for(int i = 0; i < n; i++) {
      removedLetters.addElement(getRandomLetter()); // might throw Exception
    }

    // now notify observers that we lost n letters.
    LinkFrame frame = new LinkFrame();
    frame.setGroup(BAG_GROUP);
    frame.setRequest(RQ_DECREASE_LETTER_COUNT, n); // n = n letters
    
    notifyObservers(frame);

    return removedLetters;
  }

  /**
   * Set a letter
   */
  public void setLetter(Letter letter){
    int i;
    i = (int) (letters.size()*Math.random());
    letters.insertElementAt(letter,i);
  }

  /**
   * getSize
   */

  public int getSize(){
    return letters.size();
  }
}
