/*
 * $Id: ServerBoard.java,v 1.3 1998/04/07 12:28:52 vmallet Exp $
 *
 *
 * $Log: ServerBoard.java,v $
 * Revision 1.3  1998/04/07 12:28:52  vmallet
 * Removed the putWord() method (now in Board.java)
 * Added the putWord() method calling its super and notifying observers.
 *
 * Revision 1.2  1998/04/06 09:37:05  vmallet
 * Started to add the Observable code.
 *
 * Revision 1.1  1998/04/03 16:33:15  poivert
 * Initial revision
 *
 *
 */ 

package scrobble;

import java.util.*;

import scrobble.square.*;
import game.*;



/**
 * [DESCRIBE ME]
 * herite de Board, verifie qu'un mot peut etre place sur le board, que tous les
 * mots formes sont dans le dico et compte les points.
 *
 * @version $Revision: 1.3 $
 * @author  Frederique Poivert (poivert@enst.fr)
 */
public class ServerBoard extends Board implements GameLinkConstants, ScrobbleConstants { 


  // Constructor

   /**
    * Creates a new ServerBoard
    */
  public ServerBoard(BoardShape shape) { 
    super(shape);
  }



  // Methods


  /**
   * Puts the word on the board
   * @param wordp the WordPositioner containing the word and the position of the word
   */

  public void putWord(WordPositioner wordp) {

    super.putWord(wordp);

    // notify observers
    LinkFrame frame = new LinkFrame();
    
    frame.setGroup(BOARD_GROUP);
    frame.setRequest(BOARD_PUT_WORD, NONE);
    frame.setData(wordp);

    notifyObservers(frame);
  }




  /**
   * Verifies that the given word can be put on the board, and checks all the resulting words can be found in the dictionary. 
   * In this case, this method returns the score of the player, throws exception otherwise.
   * @param wordp the WordPositioner to be verified.
   * @param dico the dictionary in which the word is to be seeked
   * @return the score of the player 
   */
  public int verifyAndCount(WordPositioner wordp, Dico dico) throws IllegalPositionException, IllegalWordException 
    {
    boolean inDico = true ;
    int score      = 0 ;
    int dir        = wordp.getDirection();
    
    //@@ a virer quand Helene aura mis dans Letter un
    //@@ constructeur sans parametre
    char c = 'c';
    Letter l       = new Letter(c);

    Vector wordsToVerify = tryAndFit(wordp);   
    //int s  = wordsToVerify.size();
    
    

    //@@ Si le dico envoie une IllegalWordException pour un mot non valide,
    //@@ le test sur inDico dans la boucle peut etre retire

    for (Enumeration e = wordsToVerify.elements(); 
                                       e.hasMoreElements(); ) {
      WordPositioner toBeChecked = (WordPositioner) e.nextElement();
	

      if (!(dico.isWord(toBeChecked))) 
	throw new IllegalWordException(toBeChecked.getWord().wordToString());
	      
    }



    // Calcul du scoring des WordPositioner contenus dans le
    // vecteur wordsToVerify :

    int sc = 0;

    for (Enumeration e = wordsToVerify.elements(); 
	 e.hasMoreElements(); ) {
      
      WordPositioner wp = (WordPositioner) e.nextElement();
     
      int w0            = wp.getWidthPosition();
      int h0            = wp.getHeightPosition();
      int nb            = wp.getWord().getVectorWord().size();
      
      
      // Calcul du scoring d'un mot vertical :
      
      if (wp.getDirection() == WordPositioner.VERTICAL) {

	// Boucle 1 pour le premier passage sur les lettres du mot
	// Appel a letterAction de Server...LetterSquare
	
	for (int i = 0; i < nb; i++ ) {
	  
	  Object obj = wp.getWord().getVectorWord().elementAt(i);
	  
	  if (obj instanceof SquarePositioner) {
	    l = (squares[h0+i][w0]).getLetter();
	    
	  } else if (obj instanceof Letter) {
	    l = (Letter) obj;
	  }
	  
	  sc = sc + ( (ServerSquare) squares[h0+i][w0] ).letterAction(l);
	  	  
	}
	
	wp.getWord().setScoring(sc);


	// Boucle 2 pour le deuxieme passage sur les lettres du mot
	// Appel a wordAction de Server...WordSquare
	
	for (int i = 0; i < nb; i++ ) {
 
	  sc = ((ServerSquare) squares[h0+i][w0]).wordAction( wp.getWord() );
	  
	}

	wp.getWord().setScoring(sc);	



      // Calcul du scoring d'un mot horizontal :

      } else if (wp.getDirection() == WordPositioner.HORIZONTAL) {

	// Boucle 1 pour le premier passage sur les lettres du mot
	// Appel a letterAction de Server...LetterSquare
	
	for (int j = 0; j < nb; j++ ) {
	  
	  Object obj = wp.getWord().getVectorWord().elementAt(j);
	  
	  if (obj instanceof SquarePositioner) {
	    l = (squares[h0][w0+j]).getLetter();
	    
	  } else if (obj instanceof Letter) {
	    l = (Letter) obj;
	  }
	  
	  sc = sc + ((ServerSquare) squares[h0][w0+j]).letterAction(l);
	  	  
	}
	
	wp.getWord().setScoring(sc);


	// Boucle 2 pour le deuxieme passage sur les lettres du mot
	// Appel a wordAction de Server...WordSquare
	
	for (int j = 0; j < nb; j++ ) {
 
	  sc = ((ServerSquare) squares[h0][w0+j]).wordAction( wp.getWord() );
	  
	}

	wp.getWord().setScoring(sc);

      }

    }

    
    // Total des points du joueur :
    // on somme les scoring de chaque mots formes

    for (Enumeration e = wordsToVerify.elements(); 
	 e.hasMoreElements(); ) {
      
      WordPositioner wp = (WordPositioner) e.nextElement();
      score = score + wp.getWord().getScoring();

    }

    return score;

  
    }


  //== Observers handling code ======================================
  

  /**
   * Adds an observer to the set of observers for this board.
   * @param obs an observer to be added.
   */
  public void addObserver(Observer obs) {
    observers.addObserver(obs);
  }

  /**
   * Deletes an observer from the set of observers for this board.
   * @param obs the observer to be deleted
   */
  public void deleteObserver(Observer obs) {
    observers.deleteObserver(obs);
  }

  /**
   * Clears the observerlist so that this board no longer has any observers.
   */
  public void deleteObservers() {
    observers.deleteObservers();
  }

  /**
   * Returns the number of observers observing this board.
   * @return the number of observers observing this board.
   */  
  public int countObservers() {
    return observers.countObservers();
  }

  /**
   * Notifies all observers that this board has changed, giving them
   * the frame containing the changes.
   *
   * @param frame the frame containing the changes
   */
  public void notifyObservers(LinkFrame frame) {
    System.out.println("<Game> notifyObservers: obs=" + observers.countObservers());
    observers.notifyObservers(frame);
  }
  

  /**
   * Set of observers that will be notified of board changes, typically the
   * <code>Player</code> objects.
   */
  protected Observable observers = new Observable() {
    public void notifyObservers(Object o) {
      setChanged();
      super.notifyObservers(o);
    }
  };


}
