
/*
 * $Id: ServerLanguage.java,v 1.2 1998/04/06 09:54:23 vmallet Exp $
 *
 * 
 * $Log: ServerLanguage.java,v $
 * Revision 1.2  1998/04/06 09:54:23  vmallet
 * Modified the debug output.
 * Changed the way of retrieving the properties file.
 *
 * Revision 1.1  1998/04/05 15:27:45  dunand
 * Initial revision
 *
 *
 */


/**
 * ServerLanguage.java
 *	
 * This class defines the <B>language</B> object on the server side.
 * 
 * @version $Revision: 1.2 $
 * @author  Loic Dunand (dunand@email.enst.fr)
 */   


package scrobble;
import game.*;
import java.util.*;
import java.net.*;


public class ServerLanguage { 

  /**
   * if true, enables the printing of debug informations on the console.
   */  
  static boolean debug = true;

  /**
   * Contains a dictionnary
   */
  Dico dictionnary;

  /**
   * Contains a bag
   */ 
  Bag bag;
  
  /**
   * Contains a distribution
   */
  Distribution distri;
  
  /**
   * The Files containing the dictionnary and the distribution with their default values
   */
  
  String dicoFile,distriFile;
  String defaultDicoFile="Little.words";
  String defaultDistriFile="test.letters";
  
  
  
  
  /**
   * The File containing the properties of the Language Object is <code>languageFile<\code>
   */
  static String languageFile = "Language.properties";
  
  /**
   * The List of valid Languages in the  <code>languageFile<\code>.
   */
  
  static Hashtable listOfLanguages	= new Hashtable();
  
  
  /** 
   *
   */
  static Properties props = new Properties();
  
  /**
   * Contains the maximum of languages that the game can handle 
   */
  static int maxLanguages=12;
  
  
  
  
  //Constructors
  /**
   * Creates the language object with an identifer.
   * @param identifier the identifier of the language to which initialise
   */
  public  ServerLanguage(int identifier) {
    
    boolean contains; 
    
    synchronized(listOfLanguages) {
      if (listOfLanguages.size()==0) loadLanguages();
    }
    synchronized(listOfLanguages) {
      contains=listOfLanguages.containsKey(new Integer(identifier));
    }
    if (contains)
      {
	synchronized(listOfLanguages) {
	  Language current = (Language) listOfLanguages.get(new Integer(identifier));
	  loadFiles(current);
	}
	dictionnary = new Dico(dicoFile);
	distri = new Distribution(distriFile);
	bag = new Bag(distri);
      }
    else {
      if (debug)
	System.out.println("<ServerLanguage> <init> - " + identifier + " : Invalid identifier !");
    }
  }
  
  
  
  // Methods

  /**
   * Gives the dictionnary
   */
  public Dico getDico(){
    return (dictionnary);
  }
  
  /**
   * Gives the bag
   */ 
  public Bag getBag(){
    return (bag);
  }
  
  /**
   * Gives the distribution
   */
  public Distribution getDistri(){
    return (distri);
  }
  
  
  
  /**
   * Gives an enum of the valid Languages in the  <code>languageFile<\code>.
   */
  static public   Enumeration getLanguages(){
    int size;
    synchronized(listOfLanguages) {
      size=listOfLanguages.size();
    }
    if (size==0) loadLanguages();
    return (listOfLanguages.elements());
  }
  
  
  /**
   * This method loads the languages in the <code>languageFile<\code> and adds them to the
   * <code>listOfLanguages</code> Vector. This function has no parameters.
   */
  
  static void loadLanguages(){
    int index;
    URL url = null;
    try {
      //      url = new URL("file","",languageFile);
      url = Class.forName("scrobble.ServerLanguage").getResource(languageFile);

    }
    catch (Exception e) {
      if (debug) 
	System.out.println("<ServerLanguage> loadLanguages(): MalformedURLException: " + url);
    }
    try {
      if (debug) 
	System.out.println("<ServerLanguage> loadLanguages(): url=" + url);
      props.load(url.openStream());
      for (Enumeration e = props.propertyNames(); e.hasMoreElements(); ) 
	{
	  String prop = (String) e.nextElement();
	  index=prop.indexOf('.');
	  String languageName = prop.substring(0,index);
	  getLanguageInfo(languageName);
	}
    } 
    catch (Exception e) {
      if (debug)
	System.out.println("<ServerLanguage> warning: " + 
			   "failed to load properties: " + url + "\nUsing default Language !");
      // we'll use defaults
      listOfLanguages.put(new Integer(1),new Language(1,"English","Default because of error loading " + url));
      return;
    }
  }
  
  /** 
   * This method gets the identifier and the description from the <code>languageFile<\code>
   */
  static void getLanguageInfo(String languageName){
    Integer identifier;
    String description;
    boolean test;
      
    try{
      identifier=Integer.valueOf(props.getProperty(languageName + ".identifier","1"),maxLanguages);
      synchronized(listOfLanguages) {
	test=listOfLanguages.containsKey(identifier);
      }
      if (!test) {
	if (debug) {
	  System.out.println("<ServerLanguage> getLanguageInfo(): Loading Language " + languageName);
	  System.out.println("   " + languageName + ".identifier = " + identifier);
	}
       	description = props.getProperty(languageName + ".description","English");
	if (debug) 
	  System.out.println("   " + languageName + ".description = " +  description);
	synchronized(listOfLanguages) {
	  listOfLanguages.put(identifier,new Language(identifier.intValue(),languageName,description));
	}
      }
    }
    catch(Exception e) {
      if (debug)
	System.out.println("<ServerLanguage> getLanguageInfo(): problem with " +  languageName + " properties"); 
    }
  }

  /**
   * This method gets the dicoFile and distriFile from the <code>languageFile<\code>
   */

  void loadFiles(Language current){
    String languageName = current.getName();
    try{
      dicoFile=props.getProperty(languageName + ".dicoFile",defaultDicoFile);
      if (debug) 
	System.out.println("<ServerLanguage> loadFiles(): " + languageName + ".dicoFile = " + dicoFile);
      distriFile=props.getProperty(languageName + ".distriFile",defaultDistriFile);
      if (debug) 
	System.out.println("<ServerLanguage> loadFiles(): " + languageName + ".distriFile = " + distriFile);
    }
      
    catch(Exception e) {
      if (debug)
	System.out.println("<ServerLanguage> loadFiles(): problem with " +  languageName + " properties");
      // Loading defaults
      dicoFile   = defaultDicoFile;
      distriFile = defaultDistriFile;
    }
  }

} 
