/*
 * $Id: ServerScrobblePlayer.java,v 1.4 1998/04/06 17:35:36 vmallet Exp $
 *
 *
 * $Log: ServerScrobblePlayer.java,v $
 * Revision 1.4  1998/04/06 17:35:36  vmallet
 * Added the playletters instanciation in constructors.
 *
 * Revision 1.3  1998/04/05 20:06:17  lozac
 * Better handling of letters transmission to client.
 *
 * Revision 1.2  1998/04/04 01:31:13  lozac
 * *** empty log message ***
 *
 * Revision 1.1  1998/04/03 18:12:30  lozac
 * Initial revision
 *
 *
 */ 

package scrobble;
import game.*;
import java.util.*;

/**
 * The <code>ServerScrobblePlayer</code> class adds the server functionnalities to <code>ScrobblePlayer</code> 
 * for the jags game, such as removing the letters of <code>ServerPlayer</code> 
 * when a word is played.
 *
 * @author  Kristell Lozac'hmeur(lozac@email.enst.fr)
 * @version $Revision: 1.4 $
 */
public class ServerScrobblePlayer extends ScrobblePlayer {
 
  // constructor

  /**
   * Creates a new ServerScrobblePlayer
   */
  public ServerScrobblePlayer() { 
   super(); 
   playletters = new Vector();
  }

  
  /**
   * creates a new ServerScrobblePlayer with his name.
   * @param name the ServerScrobblePlayer's name.
   */
  public ServerScrobblePlayer(String name) {
    super(name);
    playletters = new Vector();
  }

  /**
   * Creates a new ServerScrobblePlayer with his name and a gamelink.
   * @param name the ServerScrobblePlayer's name.
   * @param gamelink the ServerScrobblePlayer's gamelink.
   */
  public ServerScrobblePlayer(String name, GameLinkFilter gamelink) {  
    super(name, gamelink);
    playletters = new Vector();
  }
  
  
  // Methods

  /**
   * Adds a Letter in the letters of Player and increases the number of Letters of the Player by one.  
   * @param vector the Letter to be added.
   */
  public void addLetter(Letter letter) {
    LinkFrame frame;
    Vector lettervector = new Vector();

    playletters.addElement(letter); 

    sendChangement(playletters);
    
  }

  /**
   * Notify the ClientPlayer that his letters has changed and sends the new letters.
   * Notify the others Players that the number of letters of this Player has changed
   * @param letters the new letters of the Player.
   */
  public void sendChangement(Vector letters) {
    LinkFrame frame1;
    LinkFrame frame2;
    
    frame1 = createLinkframe(RQ_PLAYERCHANGELETTER, letters, getId());
    gamelink.sendBlindly(frame1); 
    frame2 = createLinkframe(RQ_PLAYERCHANGENBLETTERS, new Integer(getNbLetters()), getId());
    notifyObservers(frame2);
  }


  /**
   * adds a vector of Letters in the letters of Player, increases the number 
   * of Letters of the Player by the number of Letters added and send a frame 
   * to indicate that playletters has changed.  
   * @param vector the Letter to be added.
   */
  public void addLetters(Vector vector) {
    super.addLetters(vector);
    sendChangement(playletters);
  }

 
  /**
   * Removes all Letters from the Player's Letters
   * and sets the number of Letters of Player to zero. 
   */  
  public void removeLetters() {
    playletters.removeAllElements();
    sendChangement(playletters);
  }

  /**
   * Removes the Letters contained in vector of the Player's Letters
   * @param vector the vector of letter to be remove.
   * @exception LetterNotInRackException if a letter to be removed is not a letter of Player
   */  
  public void removeLetters(Vector letters) throws LetterNotInRackException {
    LinkFrame frame;
    Vector lettervector = new Vector() ;
    int i = 0;
    int j = 0;
    while(i < letters.size() ) {
      if( !(((Letter) letters.elementAt(i)).equals((Letter) playletters.elementAt(j))) && j < (playletters.size() - 1)) {
	j++;
      }
      else if (((Letter) letters.elementAt(i)).getName() == 
	       ((Letter) playletters.elementAt(j)).getName() ) {
	playletters.removeElementAt(j);
	i++;
	j=0;
      }
      else if (((Letter) letters.elementAt(i)).getName() != 
	       ((Letter) playletters.elementAt(j)).getName() &&
	       j == (playletters.size()-1) ) {
	i++;
	j=0;
	throw new LetterNotInRackException((Letter) letters.elementAt(i-1));	
      }
    }
    
    for (Enumeration e = getLetters() ; e.hasMoreElements() ; ) {
      lettervector.addElement( e.nextElement());
    }
    sendChangement(playletters);

  }

  

  
  public static void main(String args[]) {
    ServerScrobblePlayer x = new ServerScrobblePlayer("kristell");
    Letter letter1 = new Letter('p',3);
    Letter letter2 = new Letter('m',2);
    Letter letter3 = new Letter('h',5);
    Letter letter4 = new Letter('i',1);
    Letter letter5 = new Letter('z',10);
    Letter letter6 = new Letter('y',10);
    Letter letter7 = new Letter('u',4);
    Vector vector1 = new Vector();
    Vector vector2 = new Vector();
    int i;

    x.playletters = new Vector();
    System.out.println( " Nb de lettres initial dans playletters : " + x.getNbLetters()); 
    x.addLetter(letter1);
    x.addLetter(letter2);
    System.out.println( "Le Server joueur " + x.name + " a " + x.getNbLetters() + " lettres qui sont :" ); 
    for (i=0 ; i < x.getNbLetters() ; i++) {
      System.out.println( ((Letter) x.playletters.elementAt(i)).getName() + " qui vaut " + 
			  ((Letter) x.playletters.elementAt(i)).getValue() );
    }

    x.addLetter(letter3);
    System.out.println(" on rajoute la lettre : ");
    System.out.println( ((Letter) x.playletters.elementAt(2)).getName() + " qui vaut " + 
			((Letter) x.playletters.elementAt(2)).getValue() );
    System.out.println("Le nombre de lettres de Player est : " + x.getNbLetters());

    vector1.addElement(letter4);
    vector1.addElement(letter5);
    x.addLetters(vector1);
    System.out.println(" on rajoute les lettre : ");
    for (i=3 ; i < x.playletters.size() ;i++) {
      System.out.println( ((Letter) x.getLetterAt(i)).getName() + " qui vaut " + 
			  ((Letter) x.getLetterAt(i)).getValue() );
    }
    System.out.println("Le nombre de lettres de Player est : " + x.getNbLetters());

    System.out.println( "Le Server joueur " + x.name + " a " + x.getNbLetters() + " lettres qui sont :" ); 
    for (i=0 ; i < x.getNbLetters() ; i++) {
      System.out.println( ((Letter) x.playletters.elementAt(i)).getName() + " qui vaut " + 
			  ((Letter) x.playletters.elementAt(i)).getValue() );
    }

    vector2.addElement(letter7);
    vector2.addElement(letter2);
    vector2.addElement(letter4);
    vector2.addElement(letter6);
    vector2.addElement(letter1);
    for (i=0 ; i < vector2.size() ;i++) {  
      System.out.println("On enleve la lettre :" + ((Letter) vector2.elementAt(i)).getName() );
    }
    try {
      x.removeLetters(vector2);
    }
    catch(LetterNotInRackException e) {
      System.out.println(e);
    }
    System.out.println("il reste : ");
    for (i=0 ; i < x.getNbLetters() ; i++) {
      System.out.println( x.getLetterAt(i).getName());
    }
    System.out.println("Le nombre de lettres de Player est : " + x.getNbLetters());
    x.removeLetters();
    System.out.println("On enleve toutes les lettres");
    System.out.println("il reste " + x.getNbLetters() + " lettres"); 
  }

}




