/*
 * $Id: Word.java,v 1.5 1998/04/07 12:23:46 vmallet Exp $
 *
 *
 * $Log: Word.java,v $
 * Revision 1.5  1998/04/07 12:23:46  vmallet
 * Now implements Serializable.
 *
 * Revision 1.4  1998/04/07 10:01:03  lozac
 * added removeElement(), improved getComponent()
 *
 * Revision 1.3  1998/04/06 21:54:12  vmallet
 * Mainly fixed syntactic problems.
 *
 * Revision 1.2  1998/04/06 17:41:19  lozac
 * added getComponent(), addSquarePositioner();
 *
 * Revision 1.1  1998/04/03 16:32:45  poivert
 * Initial revision
 *
 *
 */

package scrobble;

import java.io.*; 
import java.util.*; 
import scrobble.*;

import java.awt.*;
import java.awt.event.*;
import com.sun.java.swing.*;
import com.sun.java.swing.border.*;



/**
 * Word.java
 * 
 * is a Vector of Letter or SquarePositioner.
 * There are indeed two kinds of letters : letters of player, or letters 
 * of the Board. The letters of the Board can be get with SquarePositioner.
 * @version $Revision: 1.5 $
 * @author  Frederique Poivert (poivert@enst.fr)
 */


public class Word implements Serializable {

  /**
   * List containing the ordered letters of the word or SquarePositioner.
   * There are indeed two kinds of letters : letters of player, or letters 
   * of the Board. The letters of the Board can be get with SquarePositioner.
   */
  Vector vectorWord;

  /**
   * The panel which represents the Word.
   */  
  transient JPanel wordPanel;


  /**
   * Scoring of the Word
   */
  int scoring;


  /**
   * Creates a new Word  
   */  
  public Word() {
    vectorWord = new Vector();
  }


  /**
   * Creates a new Word with the given vectorWord
   * @param vectorWord the vectorWord of the new Word being created
   */
  public Word(Vector vectorWord) {   
    this.vectorWord = vectorWord;
  }


  /**
   * Creates a new Word  with the given vectorWord and the given scoring
   * @param vectorWord the vectorWord of the new Word being created
   * @param scoring the scoring of the new Word being created
   */
  public Word(Vector vectorWord, int scoring) { 
    this.vectorWord = vectorWord;
    this.scoring = scoring;
  }



  /**
   * Modifies the vectorWord of the Word, that is : modifies the letters or
   * the SquarePositioner of the Word
   * @param newVectorWord the new vectorWord of the Word
   */
  public void setVectorWord(Vector newVectorWord) {
    vectorWord = newVectorWord;
  }


  /**
   * Returns the vectorWord of the Word
   * @return the vectorWord
   */
  public Vector getVectorWord() {
    return vectorWord;
  }

  //  /**
  //   * Returns the Letters () of Player at the specified index.
  //* @return the vector of Player's Letters.
  //   * @param index the index of the letter returned.
  //   */
  //  public Letter getLetterAt(int index) {
  //    return ((Letter) playletters.elementAt(index));
  //  }


  /**
   * Modifies the scoring of the Word
   * @param newScoring the new scoring of the Word
   */
  public void setScoring(int newScoring) {
    scoring = newScoring;
  }


  /**
   * Returns the scoring of the Word
   * @return the scoring
   */
  public int getScoring() {
    return scoring;
  }



  /**	
   * Converts a given Word into a String.
   * This method is used by the method isWord in the class Dico.
   * @return the string.
   */

  public String wordToString() {
      int s = vectorWord.size();
      char c[] = new char[s];
      for (int i = 0; i < s; i++) 
	{
	  Object objet = vectorWord.elementAt(i);
	  if (objet instanceof Letter) 
	    c[i]=((Letter) objet).getName();
	  else c[i] = ((SquarePositioner) objet).getSquare().getLetter().getName();
	}
      String str = new String(c);
      return(str);
    }



  /**
   * Adds a letter to the word at the final position
   * @param letter the letter to be added
   */

  public void addLetter(Letter letter) {

    vectorWord.addElement(letter);
    updateWordPanel();
  }

  /**
   * Adds a squarepositioner to the word at the final position
   * @param square the squarepositioner to be added
   */

  public void addSquarePositioner(SquarePositioner square) {
    vectorWord.addElement(square);
    updateWordPanel();
  }


  /**
   * Remove a letter or a squarepositionner at the position index.
   * @param index the index of the  letter or the squarepositionner to be removed.
   */  
  public void removeElementAt(int index) {
    vectorWord.removeElementAt(index);
    updateWordPanel();
  }

 /**
   * Remove a specified letter or a squarepositionner of the word.
   * @param object the object of the  letter or the squarepositionner to be removed.
   */  
  public void removeElement(Object object) {
    vectorWord.removeElement(object);
    updateWordPanel();
  }
  

  /**
   * Insert a letter at the beginning of the word
   * @param letter the letter to be inserted
   */  
    
  public void insertLetterAtFirstPosition(Letter letter) {
    vectorWord.insertElementAt(letter, 0);
    updateWordPanel();
  }

  /**
   * Returns the number of letters (or squarepositioners) vectorWord.
   */  
  public int getNbLetters() {
    return vectorWord.size();
  }

  /**
   * Returns a Panel containing the letters or squarepositioner of the word .
   */
  public Component getComponent() {
     if (wordPanel == null) {
      updateWordPanel();
        }
    
    return wordPanel;
  }


  /**
   * [DESCRIBE ME]
   */
  public void updateWordPanel() {
    boolean created = wordPanel != null;
    if (!created) {
      wordPanel = new JPanel();
      wordPanel.setLayout(new BoxLayout(wordPanel, BoxLayout.X_AXIS));
      wordPanel.setBorder(new BevelBorder(BevelBorder.LOWERED));      
    }
    wordPanel.removeAll();
    for (int i = 0 ; i < getNbLetters() ; i++) {
      wordPanel.add(((Letter) vectorWord.elementAt(i)).getComponent());
    }
    wordPanel.revalidate();
    
  }


}
